@@ -58,13 +58,11 @@ class ColumnStoreDispatcher(FileDispatcher):
             [
                 cls.deploy(
                     cls.parse,
-                    NPartitions.get() + 2,
-                    dict(
-                        fname=fname,
-                        columns=cols,
-                        num_splits=NPartitions.get(),
-                        **kwargs,
-                    ),
+                    num_returns=NPartitions.get() + 2,
+                    fname=fname,
+                    columns=cols,
+                    num_splits=NPartitions.get(),
+                    **kwargs,
                 )
                 for cols in col_partitions
             ]
@@ -136,8 +134,8 @@ class ColumnStoreDispatcher(FileDispatcher):
         else:
             row_lengths = [
                 index_chunksize
-                if i != num_partitions - 1
-                else index_len - (index_chunksize * (num_partitions - 1))
+                if (i + 1) * index_chunksize < index_len
+                else max(0, index_len - (index_chunksize * i))
                 for i in range(num_partitions)
             ]
         return index, row_lengths
